@@ -81,6 +81,7 @@ class MySQL_Thread
 	MySQL_Connection **my_idle_conns;
   bool processing_idles;
 	bool maintenance_loop;
+	bool retrieve_gtids_required; // if any of the servers has gtid_port enabled, this needs to be turned on too
 
 	PtrArray *cached_connections;
 
@@ -266,6 +267,7 @@ class MySQL_Threads_Handler
 		bool monitor_writer_is_also_reader;
 		int monitor_replication_lag_interval;
 		int monitor_replication_lag_timeout;
+		int monitor_replication_lag_count;
 		int monitor_groupreplication_healthcheck_interval;
 		int monitor_groupreplication_healthcheck_timeout;
 		int monitor_groupreplication_healthcheck_max_timeout_count;
@@ -293,6 +295,7 @@ class MySQL_Threads_Handler
 		int connect_retries_delay;
 		int connection_delay_multiplex_ms;
 		int connection_max_age_ms;
+		int connect_timeout_client;
 		int connect_timeout_server;
 		int connect_timeout_server_max;
 		int free_connections_pct;
@@ -306,7 +309,7 @@ class MySQL_Threads_Handler
 		char *interfaces;
 		char *server_version;
 		char *keep_multiplexing_variables;
-		unsigned int default_charset;
+		//unsigned int default_charset; // removed in 2.0.13 . Obsoleted previously using MySQL_Variables instead
 		unsigned int handle_unknown_charset;
 		bool servers_stats;
 		bool commands_stats;
@@ -334,6 +337,7 @@ class MySQL_Threads_Handler
 		bool use_tcp_keepalive;
 		int tcp_keepalive_time;
 		int throttle_connections_per_sec_to_hostgroup;
+		int max_transaction_idle_time;
 		int max_transaction_time;
 		int threshold_query_length;
 		int threshold_resultset_size;
@@ -394,6 +398,7 @@ class MySQL_Threads_Handler
 	} variables;
 	struct {
 		unsigned int mirror_sessions_current;
+		int threads_initialized = 0;
 	} status_variables;
 	unsigned int num_threads;
 	proxysql_mysql_thread_t *mysql_threads;
