@@ -3,6 +3,7 @@
 #include <stack>
 
 #include <algorithm>
+#include <set>
 
 #ifndef EZOPTION
 #define EZOPTION
@@ -49,11 +50,6 @@
 
 #include <sys/ioctl.h>
 
-#if !defined(__FreeBSD__) && !defined(__APPLE__)
-#define HAVE_BOOL
-#include "ma_global.h"
-//#include "my_pthread.h"
-#endif
 #include "mysql.h"
 #include "mariadb_com.h"
 
@@ -83,7 +79,7 @@
 #else
 #define DEB ""
 #endif /* DEBUG */
-#define PROXYSQL_VERSION	"2.0.12-percona-1.1" DEB
+#define PROXYSQL_VERSION	"2.0.17-percona-1.1" DEB
 #define PROXYSQL_CODENAME	"Truls"
 
 #ifndef PROXYSQL_FUNC_DEFS
@@ -111,6 +107,8 @@ void print_backtrace(void);
 void init_debug_struct();
 void init_debug_struct_from_cmdline();
 void proxy_debug_func(enum debug_module, int, int, const char *, int, const char *, const char *, ...);
+void proxy_debug_get_filters(std::set<std::string>&);
+void proxy_debug_load_filters(std::set<std::string>&);
 #endif
 
 #ifdef __cplusplus
