@@ -86,6 +86,38 @@ void MySQL_Variables::server_set_hash_and_value(MySQL_Session* session, int idx,
 }
 
 
+/**
+ * @brief Set the supplied value for the session variable specified by the supplied
+ *  index into the supplied client session.
+ *
+ * @details There are two session variables which require special handling:
+ *   - 'SET NAMES'
+ *   - 'SET CHARACTER SET'
+ *
+ * For the second case 'SET CHARACTER SET' we forget about the values for:
+ *   - 'SQL_CHARACTER_SET_CONNECTION'
+ *   - 'SQL_COLLATION_CONNECTION'
+ *
+ * This is done because 'character_set_database' is not known when the set happens and
+ * because we work under the assumption that if a client request 'SET CHARACTER SET'
+ * doesn't require a specific 'collation_connection' and 'character_set_connection".
+ * Furthermore, 'character_set_database' is deprecated since MySQL 5.7 and will only
+ * be usable as an immutable session variable in the future. For reference see:
+ *
+ * - 'https://dev.mysql.com/doc/refman/5.7/en/server-system-variables.html#sysvar_character_set_database
+ *
+ * Due to this, *it's expected behavior* to see that a connection that sets 'SET CHARACTER SET'
+ * has a variant 'collation_connection' and 'character_set_connection' depending on the
+ * backend connection that is retrieved from the connection pool. If the 'collation_connection'
+ * and 'character_set_connection' variables are relevant and should never change,
+ * 'SET NAMES' should be used.
+ *
+ * @param session The client session which variable value is going to be modified.
+ * @param idx The index of the session variable to modify.
+ * @param value The session variable value to be set.
+ *
+ * @return 'true' in case of success, 'false' otherwise.
+ */
 bool MySQL_Variables::client_set_value(MySQL_Session* session, int idx, const std::string& value) {
 	if (!session || !session->client_myds || !session->client_myds->myconn) {
 		proxy_warning("Session validation failed\n");
@@ -243,7 +275,11 @@ bool validate_charset(MySQL_Session* session, int idx, int &_rc) {
 					if (idx == SQL_COLLATION_CONNECTION) {
 						ci = proxysql_find_charset_collate(mysql_thread___default_variables[idx]);
 					} else {
-						ci = proxysql_find_charset_name(mysql_thread___default_variables[idx]);
+						if (mysql_thread___default_variables[idx]) {
+							ci = proxysql_find_charset_name(mysql_thread___default_variables[idx]);
+						} else {
+							ci = proxysql_find_charset_name(mysql_thread___default_variables[SQL_CHARACTER_SET]);
+						}
 					}
 
 					if (!ci) {
@@ -265,7 +301,11 @@ bool validate_charset(MySQL_Session* session, int idx, int &_rc) {
 					if (idx == SQL_COLLATION_CONNECTION) {
 						ci = proxysql_find_charset_collate(mysql_thread___default_variables[idx]);
 					} else {
-						ci = proxysql_find_charset_name(mysql_thread___default_variables[idx]);
+						if (mysql_thread___default_variables[idx]) {
+							ci = proxysql_find_charset_name(mysql_thread___default_variables[idx]);
+						} else {
+							ci = proxysql_find_charset_name(mysql_thread___default_variables[SQL_CHARACTER_SET]);
+						}
 					}
 
 					if (!ci) {
@@ -451,7 +491,7 @@ bool logbin_update_server_variable(MySQL_Session* session, int idx, int &_rc) {
 }
 
 
-bool MySQL_Variables::parse_variable_boolean(MySQL_Session *sess, int idx, string& value1, bool& exit_after_SetParse, bool * lock_hostgroup) {
+bool MySQL_Variables::parse_variable_boolean(MySQL_Session *sess, int idx, string& value1, bool * lock_hostgroup) {
 	proxy_debug(PROXY_DEBUG_MYSQL_COM, 5, "Processing SET %s value %s\n", mysql_tracked_variables[idx].set_variable_name, value1.c_str());
 	int __tmp_value = -1;
 	if (
@@ -483,7 +523,7 @@ bool MySQL_Variables::parse_variable_boolean(MySQL_Session *sess, int idx, strin
 			}
 			proxy_debug(PROXY_DEBUG_MYSQL_COM, 5, "Changing connection %s to %s\n", mysql_tracked_variables[idx].set_variable_name, value1.c_str());
 		}
-		exit_after_SetParse = true;
+		//exit_after_SetParse = true;
 	} else {
 		sess->unable_to_parse_set_statement(lock_hostgroup);
 		return false;
@@ -493,7 +533,7 @@ bool MySQL_Variables::parse_variable_boolean(MySQL_Session *sess, int idx, strin
 
 
 
-bool MySQL_Variables::parse_variable_number(MySQL_Session *sess, int idx, string& value1, bool& exit_after_SetParse, bool * lock_hostgroup) {
+bool MySQL_Variables::parse_variable_number(MySQL_Session *sess, int idx, string& value1, bool * lock_hostgroup) {
 	int vl = strlen(value1.c_str());
 	const char *v = value1.c_str();
 	bool only_digit_chars = true;
@@ -517,7 +557,7 @@ bool MySQL_Variables::parse_variable_number(MySQL_Session *sess, int idx, string
 				return false;
 			proxy_debug(PROXY_DEBUG_MYSQL_COM, 5, "Changing connection %s to %s\n", mysql_tracked_variables[idx].set_variable_name, value1.c_str());
 		}
-		exit_after_SetParse = true;
+		//exit_after_SetParse = true;
 	} else {
 		sess->unable_to_parse_set_statement(lock_hostgroup);
 		return false;
