@@ -9,27 +9,35 @@
 #include "query_processor.h"
 #include "MySQL_Variables.h"
 
+#include <atomic>
+
+extern char * binary_sha1;
+
 extern const MARIADB_CHARSET_INFO * proxysql_find_charset_nr(unsigned int nr);
 MARIADB_CHARSET_INFO * proxysql_find_charset_name(const char *name);
 
 void Variable::fill_server_internal_session(json &j, int conn_num, int idx) {
 	if (idx == SQL_CHARACTER_SET_RESULTS || idx == SQL_CHARACTER_SET_CLIENT || idx == SQL_CHARACTER_SET_DATABASE) {
 		const MARIADB_CHARSET_INFO *ci = NULL;
-		if (!value)
+		if (!value) {
 			ci = proxysql_find_charset_name(mysql_tracked_variables[idx].default_value);
-		else
+		} else if (strcasecmp("NULL", value) && strcasecmp("binary", value)) {
 			ci = proxysql_find_charset_nr(atoi(value));
+		}
 		if (!ci) {
 			if (idx == SQL_CHARACTER_SET_RESULTS && (!strcasecmp("NULL", value) || !strcasecmp("binary", value))) {
-				j["conn"][mysql_tracked_variables[idx].internal_variable_name] = (ci && ci->csname)?ci->csname:"";
-			}
-			else {
+				if (!strcasecmp("NULL", value)) {
+					j["backends"][conn_num]["conn"][mysql_tracked_variables[idx].internal_variable_name] = "";
+				} else {
+					j["backends"][conn_num]["conn"][mysql_tracked_variables[idx].internal_variable_name] = value;
+				}
+			} else {
 				proxy_error("Cannot find charset [%s] for variables %d\n", value, idx);
 				assert(0);
 			}
+		} else {
+			j["backends"][conn_num]["conn"][mysql_tracked_variables[idx].internal_variable_name] = std::string((ci && ci->csname)?ci->csname:"");
 		}
-
-		j["backends"][conn_num]["conn"][mysql_tracked_variables[idx].internal_variable_name] = std::string((ci && ci->csname)?ci->csname:"");
 	} else if (idx == SQL_CHARACTER_SET_CONNECTION) {
 		const MARIADB_CHARSET_INFO *ci = NULL;
 		if (!value)
@@ -59,25 +67,31 @@ void Variable::fill_server_internal_session(json &j, int conn_num, int idx) {
 void Variable::fill_client_internal_session(json &j, int idx) {
 	if (idx == SQL_CHARACTER_SET_RESULTS || idx == SQL_CHARACTER_SET_CLIENT || idx == SQL_CHARACTER_SET_DATABASE) {
 		const MARIADB_CHARSET_INFO *ci = NULL;
-		if (!value)
+		if (!value) {
 			ci = proxysql_find_charset_name(mysql_tracked_variables[idx].default_value);
-		else
+		} else if (strcasecmp("NULL", value) && strcasecmp("binary", value)) {
 			ci = proxysql_find_charset_nr(atoi(value));
+		}
 		if (!ci) {
 			if (idx == SQL_CHARACTER_SET_RESULTS && (!strcasecmp("NULL", value) || !strcasecmp("binary", value))) {
-				j["conn"][mysql_tracked_variables[idx].internal_variable_name] = (ci && ci->csname)?ci->csname:"";
-			}
-			else {
+				if (!strcasecmp("NULL", value)) {
+					j["conn"][mysql_tracked_variables[idx].internal_variable_name] = "";
+				} else {
+					j["conn"][mysql_tracked_variables[idx].internal_variable_name] = value;
+				}
+			} else {
 				proxy_error("Cannot find charset [%s] for variables %d\n", value, idx);
 				assert(0);
 			}
+		} else {
+			j["conn"][mysql_tracked_variables[idx].internal_variable_name] = (ci && ci->csname)?ci->csname:"";
 		}
-
-		j["conn"][mysql_tracked_variables[idx].internal_variable_name] = (ci && ci->csname)?ci->csname:"";
-
 	} else if (idx == SQL_CHARACTER_SET_CONNECTION) {
 		const MARIADB_CHARSET_INFO *ci = NULL;
-		ci = proxysql_find_charset_nr(atoi(value));
+		if (!value)
+			ci = proxysql_find_charset_collate(mysql_tracked_variables[idx].default_value);
+		else
+			ci = proxysql_find_charset_nr(atoi(value));
 		j["conn"][mysql_tracked_variables[idx].internal_variable_name] = (ci && ci->csname)?ci->csname:"";
 	} else if (idx == SQL_COLLATION_CONNECTION) {
 		const MARIADB_CHARSET_INFO *ci = NULL;
@@ -509,6 +523,14 @@ void MySQL_Connection::set_status_user_variable(bool v) {
 	}
 }
 
+void MySQL_Connection::set_status_has_savepoint(bool v) {
+	if (v) {
+		status_flags |= STATUS_MYSQL_CONNECTION_HAS_SAVEPOINT;
+	} else {
+		status_flags &= ~STATUS_MYSQL_CONNECTION_HAS_SAVEPOINT;
+	}
+}
+
 void MySQL_Connection::set_status_prepared_statement(bool v) {
 	if (v) {
 		status_flags |= STATUS_MYSQL_CONNECTION_PREPARED_STATEMENT;
@@ -548,6 +570,10 @@ bool MySQL_Connection::get_status_user_variable() {
 	return status_flags & STATUS_MYSQL_CONNECTION_USER_VARIABLE;
 }
 
+bool MySQL_Connection::get_status_has_savepoint() {
+	return status_flags & STATUS_MYSQL_CONNECTION_HAS_SAVEPOINT;
+}
+
 bool MySQL_Connection::get_status_get_lock() {
 	return status_flags & STATUS_MYSQL_CONNECTION_GET_LOCK;
 }
@@ -580,7 +606,41 @@ bool MySQL_Connection::get_status_sql_log_bin0() {
 	return status_flags & STATUS_MYSQL_CONNECTION_SQL_LOG_BIN0;
 }
 
-bool MySQL_Connection::match_tracked_options(MySQL_Connection *c) {
+bool MySQL_Connection::requires_CHANGE_USER(const MySQL_Connection *client_conn) {
+	char *username = client_conn->userinfo->username;
+	if (strcmp(userinfo->username,username)) {
+		// the two connections use different usernames
+		// The connection need to be reset with CHANGE_USER
+		return true;
+	}
+	for (auto i = 0; i < SQL_NAME_LAST; i++) {
+		if (client_conn->var_hash[i] == 0) {
+			if (var_hash[i]) {
+				// this connection has a variable set that the
+				// client connection doesn't have.
+				// Since connection cannot be unset , this connection
+				// needs to be reset with CHANGE_USER
+				return true;
+			}
+		}
+	}
+	return false;
+}
+unsigned int MySQL_Connection::number_of_matching_session_variables(const MySQL_Connection *client_conn, unsigned int& not_matching) {
+	unsigned int ret=0;
+	for (auto i = 0; i < SQL_NAME_LAST; i++) {
+		if (client_conn->var_hash[i] && i != SQL_CHARACTER_ACTION) { // client has a variable set
+			if (var_hash[i] == client_conn->var_hash[i]) { // server conection has the variable set to the same value
+				ret++;
+			} else {
+				not_matching++;
+			}
+		}
+	}
+	return ret;
+}
+
+bool MySQL_Connection::match_tracked_options(const MySQL_Connection *c) {
 	uint32_t cf1 = options.client_flag; // own client flags
 	uint32_t cf2 = c->options.client_flag; // other client flags
 	if ((cf1 & CLIENT_FOUND_ROWS) == (cf2 & CLIENT_FOUND_ROWS)) {
@@ -603,6 +663,28 @@ void MySQL_Connection::connect_start() {
 	mysql_options(mysql, MYSQL_OPT_NONBLOCK, 0);
 	mysql_options4(mysql, MYSQL_OPT_CONNECT_ATTR_ADD, "program_name", "proxysql");
 	mysql_options4(mysql, MYSQL_OPT_CONNECT_ATTR_ADD, "_server_host", parent->address);
+	{
+		time_t __timer;
+		char __buffer[25];
+		struct tm *__tm_info;
+		time(&__timer);
+		__tm_info = localtime(&__timer);
+		strftime(__buffer, 25, "%Y-%m-%d %H:%M:%S", __tm_info);
+		mysql_options4(mysql, MYSQL_OPT_CONNECT_ATTR_ADD, "connection_creation_time", __buffer);
+		unsigned long long t1=monotonic_time();
+		sprintf(__buffer,"%llu",(t1-GloVars.global.start_time)/1000/1000);
+		mysql_options4(mysql, MYSQL_OPT_CONNECT_ATTR_ADD, "proxysql_uptime", __buffer);
+		sprintf(__buffer,"%d", parent->myhgc->hid);
+		mysql_options4(mysql, MYSQL_OPT_CONNECT_ATTR_ADD, "hostgroup_id", __buffer);
+		mysql_options4(mysql, MYSQL_OPT_CONNECT_ATTR_ADD, "compile_time", __TIMESTAMP__);
+		mysql_options4(mysql, MYSQL_OPT_CONNECT_ATTR_ADD, "proxysql_version", PROXYSQL_VERSION);
+		if (binary_sha1) {
+			mysql_options4(mysql, MYSQL_OPT_CONNECT_ATTR_ADD, "proxysql_sha1", binary_sha1);
+		} else {
+			mysql_options4(mysql, MYSQL_OPT_CONNECT_ATTR_ADD, "proxysql_sha1", "unknown");
+		}
+		mysql_options4(mysql, MYSQL_OPT_CONNECT_ATTR_ADD, "mysql_bug_102266", "ProxySQL is sending a lot of data to MySQL server using CLIENT_CONNECT_ATTRS in order to not hit MySQL bug https://bugs.mysql.com/bug.php?id=102266 . See also https://github.com/sysown/proxysql/issues/3276");
+	}
 	if (parent->use_ssl) {
 		mysql_ssl_set(mysql, mysql_thread___ssl_p2s_key, mysql_thread___ssl_p2s_cert, mysql_thread___ssl_p2s_ca, NULL, mysql_thread___ssl_p2s_cipher);
 	}
@@ -632,12 +714,14 @@ void MySQL_Connection::connect_start() {
 		std::stringstream ss;
 		ss << c->nr;
 
+		mysql_variables.server_set_value(myds->sess, SQL_CHARACTER_SET, ss.str().c_str());
 		mysql_variables.server_set_value(myds->sess, SQL_CHARACTER_SET_RESULTS, ss.str().c_str());
 		mysql_variables.server_set_value(myds->sess, SQL_CHARACTER_SET_CLIENT, ss.str().c_str());
 		mysql_variables.server_set_value(myds->sess, SQL_CHARACTER_SET_CONNECTION, ss.str().c_str());
 		mysql_variables.server_set_value(myds->sess, SQL_COLLATION_CONNECTION, ss.str().c_str());
 	}
-	mysql_options(mysql, MYSQL_SET_CHARSET_NAME, c->csname);
+	//mysql_options(mysql, MYSQL_SET_CHARSET_NAME, c->csname);
+	mysql->charset = c;
 	unsigned long client_flags = 0;
 	//if (mysql_thread___client_found_rows)
 	//	client_flags += CLIENT_FOUND_ROWS;
@@ -708,6 +792,11 @@ void MySQL_Connection::change_user_start() {
 			auth_password=userinfo->password;
 		}
 	}
+	// we first reset the charset to a default one.
+	// this to solve the problem described here:
+	// https://github.com/sysown/proxysql/pull/3249#issuecomment-761887970
+	if (mysql->charset->nr >= 255)
+		mysql_options(mysql, MYSQL_SET_CHARSET_NAME, mysql->charset->csname);
 	async_exit_status = mysql_change_user_start(&ret_bool,mysql,_ui->username, auth_password, _ui->schemaname);
 }
 
@@ -1557,15 +1646,9 @@ int MySQL_Connection::async_query(short event, char *stmt, unsigned long length,
 	assert(mysql);
 	assert(ret_mysql);
 	server_status=parent->status; // we copy it here to avoid race condition. The caller will see this
-	if (
-		(server_status==MYSQL_SERVER_STATUS_OFFLINE_HARD) // the server is OFFLINE as specific by the user
-		||
-		(server_status==MYSQL_SERVER_STATUS_SHUNNED && parent->shunned_automatic==true && parent->shunned_and_kill_all_connections==true) // the server is SHUNNED due to a serious issue
-		||
-		(server_status==MYSQL_SERVER_STATUS_SHUNNED_REPLICATION_LAG) // slave is lagging! see #774
-	) {
+	if (IsServerOffline())
 		return -1;
-	}
+
 	if (myds) {
 		if (myds->DSS != STATE_MARIADB_QUERY) {
 			myds->DSS = STATE_MARIADB_QUERY;
@@ -1691,6 +1774,10 @@ int MySQL_Connection::async_change_user(short event) {
 	PROXY_TRACE();
 	assert(mysql);
 	assert(ret_mysql);
+	server_status=parent->status; // we copy it here to avoid race condition. The caller will see this
+	if (IsServerOffline())
+		return -1;
+
 	switch (async_state_machine) {
 		case ASYNC_CHANGE_USER_SUCCESSFUL:
 			unknown_transaction_status = false;
@@ -1734,6 +1821,10 @@ int MySQL_Connection::async_select_db(short event) {
 	PROXY_TRACE();
 	assert(mysql);
 	assert(ret_mysql);
+	server_status=parent->status; // we copy it here to avoid race condition. The caller will see this
+	if (IsServerOffline())
+		return -1;
+
 	switch (async_state_machine) {
 		case ASYNC_INITDB_SUCCESSFUL:
 			unknown_transaction_status = false;
@@ -1771,6 +1862,10 @@ int MySQL_Connection::async_set_autocommit(short event, bool ac) {
 	PROXY_TRACE();
 	assert(mysql);
 	assert(ret_mysql);
+	server_status=parent->status; // we copy it here to avoid race condition. The caller will see this
+	if (IsServerOffline())
+		return -1;
+
 	switch (async_state_machine) {
 		case ASYNC_SET_AUTOCOMMIT_SUCCESSFUL:
 			unknown_transaction_status = false;
@@ -1810,6 +1905,10 @@ int MySQL_Connection::async_set_names(short event, unsigned int c) {
 	PROXY_TRACE();
 	assert(mysql);
 	assert(ret_mysql);
+	server_status=parent->status; // we copy it here to avoid race condition. The caller will see this
+	if (IsServerOffline())
+		return -1;
+
 	switch (async_state_machine) {
 		case ASYNC_SET_NAMES_SUCCESSFUL:
 			unknown_transaction_status = false;
@@ -1849,6 +1948,10 @@ int MySQL_Connection::async_set_option(short event, bool mask) {
 	PROXY_TRACE();
 	assert(mysql);
 	assert(ret_mysql);
+	server_status=parent->status; // we copy it here to avoid race condition. The caller will see this
+	if (IsServerOffline())
+		return -1;
+
 	switch (async_state_machine) {
 		case ASYNC_SET_OPTION_SUCCESSFUL:
 			unknown_transaction_status = false;
@@ -1942,6 +2045,12 @@ bool MySQL_Connection::IsActiveTransaction() {
 				ret = true;
 			}
 		}
+		if (ret == false) {
+			if (get_status_has_savepoint()) {
+				// there are savepoints
+				ret = true;
+			}
+		}
 	}
 	return ret;
 }
@@ -1974,7 +2083,7 @@ bool MySQL_Connection::MultiplexDisabled() {
 // status_flags stores information about the status of the connection
 // can be used to determine if multiplexing can be enabled or not
 	bool ret=false;
-	if (status_flags & (STATUS_MYSQL_CONNECTION_TRANSACTION|STATUS_MYSQL_CONNECTION_USER_VARIABLE|STATUS_MYSQL_CONNECTION_PREPARED_STATEMENT|STATUS_MYSQL_CONNECTION_LOCK_TABLES|STATUS_MYSQL_CONNECTION_TEMPORARY_TABLE|STATUS_MYSQL_CONNECTION_GET_LOCK|STATUS_MYSQL_CONNECTION_NO_MULTIPLEX|STATUS_MYSQL_CONNECTION_SQL_LOG_BIN0|STATUS_MYSQL_CONNECTION_FOUND_ROWS|STATUS_MYSQL_CONNECTION_NO_BACKSLASH_ESCAPES) ) {
+	if (status_flags & (STATUS_MYSQL_CONNECTION_TRANSACTION|STATUS_MYSQL_CONNECTION_USER_VARIABLE|STATUS_MYSQL_CONNECTION_PREPARED_STATEMENT|STATUS_MYSQL_CONNECTION_LOCK_TABLES|STATUS_MYSQL_CONNECTION_TEMPORARY_TABLE|STATUS_MYSQL_CONNECTION_GET_LOCK|STATUS_MYSQL_CONNECTION_NO_MULTIPLEX|STATUS_MYSQL_CONNECTION_SQL_LOG_BIN0|STATUS_MYSQL_CONNECTION_FOUND_ROWS|STATUS_MYSQL_CONNECTION_NO_BACKSLASH_ESCAPES|STATUS_MYSQL_CONNECTION_HAS_SAVEPOINT) ) {
 		ret=true;
 	}
 	if (auto_increment_delay_token) return true;
@@ -2159,6 +2268,35 @@ void MySQL_Connection::ProcessQueryAndSetStatusFlags(char *query_digest_text) {
 			set_status_found_rows(true);
 		}
 	}
+	if (get_status_has_savepoint()==false) {
+		if (mysql) {
+			if (
+				(mysql->server_status & SERVER_STATUS_IN_TRANS)
+				||
+				((mysql->server_status & SERVER_STATUS_AUTOCOMMIT) == 0)
+			) {
+				if (!strncasecmp(query_digest_text,"SAVEPOINT ", strlen("SAVEPOINT "))) {
+					set_status_has_savepoint(true);
+				}
+			}
+		}
+	} else {
+		if (
+			(
+				// make sure we don't have a transaction running
+				// checking just for COMMIT and ROLLBACK is not enough, because `SET autocommit=1` can commit too
+				(mysql->server_status & SERVER_STATUS_AUTOCOMMIT)
+				&&
+				( (mysql->server_status & SERVER_STATUS_IN_TRANS) == 0 )
+			)
+			||
+			!strcasecmp(query_digest_text,"COMMIT")
+			||
+			!strcasecmp(query_digest_text,"ROLLBACK")
+		) {
+			set_status_has_savepoint(false);
+		}
+	}
 }
 
 void MySQL_Connection::optimize() {
@@ -2205,6 +2343,7 @@ int MySQL_Connection::async_send_simple_command(short event, char *stmt, unsigne
 	PROXY_TRACE();
 	assert(mysql);
 	assert(ret_mysql);
+	server_status=parent->status; // we copy it here to avoid race condition. The caller will see this
 	if (
 		(parent->status==MYSQL_SERVER_STATUS_OFFLINE_HARD) // the server is OFFLINE as specific by the user
 		||
