@@ -432,6 +432,11 @@ int MySQL_Data_Stream::read_from_net() {
 		//proxy_info("Entering\n");
 	}
 	if ((revents & POLLIN)==0) return 0;
+	if (revents & POLLHUP) {
+		shut_soft();
+		return -1;
+	}
+
 	int r=0;
 	int s=queue_available(queueIN);
 	if (encrypted) {
@@ -562,6 +567,11 @@ int MySQL_Data_Stream::read_from_net() {
 		} else {
 			int ssl_ret=SSL_get_error(ssl, r);
 			if (ssl_ret!=SSL_ERROR_WANT_READ && ssl_ret!=SSL_ERROR_WANT_WRITE) shut_soft();
+			if (r==0 && revents==1) {
+				// revents returns 1 , but recv() returns 0 , so there is no data.
+				// Therefore the socket is already closed
+				shut_soft();
+			}
 		}
 	} else {
 		queue_w(queueIN,r);
